package wafly.app.remotedesktop2.core;

import java.awt.image.BufferedImage;
import java.io.File;
import java.util.HashMap;
import wafly.app.remotedesktop2.Program;
import wafly.app.remotedesktop2.comm.Config;
import wafly.app.remotedesktop2.comm.Util;
import wafly.app.remotedesktop2.forms.MainForm;
import wafly.tool.util.draw.ImageUtil;
import wafly.tool.util.draw.ImageUtil.ImageType;
import wafly.tool.util.file.FileUtil;
import wafly.tool.util.os.KeyboardUtil;
import wafly.tool.util.os.MouseUtil;
import wafly.tool.util.ui.ScreenUtil;
import wafly.tool.util.web.WebService;

public class Listener extends WebService {
	
	private MouseUtil mouseUtil = new MouseUtil();
	private KeyboardUtil keyboardUtil = new KeyboardUtil();

	public Listener(int port) {
		super(port);
	}

	@Override
	public void startService() {
		super.startService();
		Util.log("服务启动");
	}

	@Override
	public void stopService() {
		super.stopService();
		Util.log("服务停止");
	}

	@Override
	public ContentType getDataType(String host, String url, HashMap<String, String> params) {
		if(url.equals(Config.URL_SCREEN)){
			return ContentType.FLASH;
		}else if(url.startsWith(Config.URL_IMG)){
			return ContentType.JPEG;
		}else if(url.equals(Config.URL_HOME) && Config.aotuFit){
			return ContentType.FLASH;
		}else if(url.equals(Config.URL_DOWNLOAD)){
			return ContentType.ATTACHMENT;
		}
		return ContentType.TEXT;
	}

	@Override
	public byte[] getData(String host, String url, HashMap<String, String> params, int[] postContents, FileDataRemark remark) {
		if(url.equals(Config.URL_HOME)){
			Util.log("用户["+host+"]登入");
			if(Config.aotuFit)
				return new FileUtil().getContent(Config.PATH_FLASH);
			else
				return Program.ScreenSource.getBytes();
		}else if(url.startsWith(Config.URL_IMG)){
			return ImageUtil.getImageData(
					ImageUtil.getThumbnail((BufferedImage)ScreenUtil.getScreenDump(), (int)(Config.screenSize*Config.screenScale)),
					ImageType.JPG);
		}else if(url.equals(Config.URL_SCREEN)){
			return new FileUtil().getContent(Config.PATH_FLASH);
		}else if(url.startsWith(Config.URL_KEY_DOWN)){
			if(!Config.isViewMode){
				String keyCode=url.substring(url.indexOf("=")+1, url.indexOf("&"));
				keyboardUtil.keyDown(Util.getKeyCode(Integer.parseInt(keyCode)));
			}
		}else if(url.startsWith(Config.URL_KEY_UP)){
			if(!Config.isViewMode){
				String keyCode=url.substring(url.indexOf("=")+1, url.indexOf("&"));
				keyboardUtil.keyUp(Util.getKeyCode(Integer.parseInt(keyCode)));
			}
		}else if(url.startsWith(Config.URL_MOVE)){
			if(!Config.isViewMode){
				String[] pos = url.substring(url.indexOf("=")+1).split(",");
				int x = (int)(Double.parseDouble(pos[0])*Config.sSize.width/100)+1;
				int y = (int)(Double.parseDouble(pos[1])*Config.sSize.height/100)+1;
				mouseUtil.mouseMoveTo(x, y);
			}
		}else if(url.startsWith(Config.URL_LEFT_DOWN)){
			if(!Config.isViewMode){
				mouseUtil.lMouseDown();
			}
		}else if(url.startsWith(Config.URL_LEFT_UP)){
			if(!Config.isViewMode){
				mouseUtil.lMouseUp();
			}
		}else if(url.startsWith(Config.URL_RIGHT_CLICK)){
			if(!Config.isViewMode){
				mouseUtil.rMouseDown();
				mouseUtil.rMouseUp();
			}
		}else if(url.startsWith(Config.URL_WHEEL_DOWN)){
			if(!Config.isViewMode){
				mouseUtil.mouseWheelDown(1);
			}
		}else if(url.startsWith(Config.URL_WHEEL_UP)){
			if(!Config.isViewMode){
				mouseUtil.mouseWheelUp(1);
			}
		}else if(url.equals(Config.URL_DOWNLOAD)){
			/* TODO
			if(MainForm.dlPath==null){
				return null;
			}
			File file = new File(MainForm.dlPath);
			byte[] filec = null;
			if(file.exists()){
				filec = new FileUtil().getContent(MainForm.dlPath);
				remark.remark=file.getName();
			}
			MainForm.dlPath=null;
			MainForm.infoLabel.setText("将要下载的文件拖放到此处。");
			return filec;
			*/
			return null;
		}
		return new byte[0];
	}

	@Override
	public String getReturnFileName(FileDataRemark remark) {
		return remark.remark;
	}

}
