package wafly.app.remotedesktop2.forms;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.TrayIcon;
import java.awt.Window;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import wafly.app.remotedesktop2.Program;
import wafly.app.remotedesktop2.comm.Config;
import wafly.app.remotedesktop2.core.Listener;
import wafly.control.textinput.NumberField;
import wafly.control.textinput.ValueChangeListener;
import wafly.tool.util.file.FileUtil;
import wafly.tool.util.process.RuntimeUtil;
import wafly.tool.util.web.NetworkUtil;

public class MainForm extends JPanel {
	
	private static final long serialVersionUID = 1L;
	private boolean isServicing=false;
	private Window container=null;

	public static DefaultListModel model = new DefaultListModel();
	public static String dlPath = null;
	public static JLabel infoLabel = new JLabel("将要下载的文件拖放到此处。");
	private JButton startServiceBtn = new JButton("启动服务");
	private NumberField portValue = new NumberField();
	private JLabel lanIPValueLabel = new JLabel("正在获取网卡IP");
	private JCheckBox isStartOnInitCheckBox = new JCheckBox("启动后自动开启服务");
	private JCheckBox isMinOnInitCheckBox = new JCheckBox("启动后自动最小化");
//	private JCheckBox isSingleUserCheckBox = new JCheckBox("单用户模式");
	private JCheckBox isShowInTaskCheckBox = new JCheckBox("在任务栏显示");
//	private JCheckBox isSoundCheckBox = new JCheckBox("开启声音");
	private JCheckBox isShowTrayCheckBox = new JCheckBox("显示托盘图标");
	private TrayIcon tray = null;
	private JComboBox closeCombox = new JComboBox(new String[]{"询问","最小化窗口","退出程序"});
	private JCheckBox sizeCheckBox = new JCheckBox("指定屏幕显示尺寸：");
	private NumberField widthNumber = new NumberField();
	private NumberField heightNumber = new NumberField();
	private JCheckBox isViewOnlyCheckBox = new JCheckBox("观察者模式");
	private JSlider queSlider = new JSlider (JSlider.HORIZONTAL, 10, 100, 100);
	private JLabel quiteValueLabel = new JLabel();
	
	public MainForm(Window container){
		this.container=container;
		this.setLayout(null);
		JTabbedPane tabPanels = new JTabbedPane();
//		tabPanels.setTabPlacement(JTabbedPane.RIGHT);
		tabPanels.setSize(300, 190);
		tabPanels.setLocation(5, 5);
		tabPanels.addTab("控制", new ControlPanel());
		tabPanels.addTab("帮助", new AboutPanel());
		tabPanels.addTab("服务设置", new SysSettingPanel());
		tabPanels.addTab("显示设置", new ViewSettingPanel());
		tabPanels.addTab("下载槽", new DownloadPanel());
		tabPanels.addTab("关于...", new InfoPanel());
		this.add(tabPanels);
		new Thread(){public void run(){
			controlValueInit();
		}}.start();
	}
	
	private void controlValueInit(){
		String[] IPs = NetworkUtil.getIPs(1);
		StringBuffer ipSB = new StringBuffer("<html>");
		for(int i=0; i<IPs.length; i++){
			if(IPs[i].equals("127.0.0.1"))
				continue;
			if(i==3 && IPs.length>4)
				ipSB.append("...<br>");
			else
				ipSB.append(IPs[i]).append("<br>");
		}
		ipSB.append("</html>");
		lanIPValueLabel.setText(ipSB.toString());
		portValue.setNumber(Config.port);
		if(Config.miniOnInit){
			isMinOnInitCheckBox.setSelected(true);
			if(container instanceof JFrame)
				((JFrame)this.container).setState(Frame.ICONIFIED);
			else
				this.setVisible(false);
		}
//		isSingleUserCheckBox.setSelected(Config.isSingleUser);
		isShowInTaskCheckBox.setSelected(Config.showInTask);
//		if(Config.useSound){
//			isSoundCheckBox.setSelected(true);
//		}
		if(Config.useTray){
			isShowTrayCheckBox.setSelected(true);
			if(tray==null){
				tray = RuntimeUtil.addTrayIcon(new File(Config.PATH_LOGO));
				trayOperation();
			}
		}
		closeCombox.setSelectedIndex(Config.actionOnClose);
		sizeCheckBox.setSelected(!Config.aotuFit);
		widthNumber.setNumber(Config.showSrcWidth);
		widthNumber.setEnabled(sizeCheckBox.isSelected());
		heightNumber.setNumber(Config.showSrcHeight);
		heightNumber.setEnabled(sizeCheckBox.isSelected());
		isViewOnlyCheckBox.setSelected(Config.isViewMode);
		queSlider.setValue((int)(Config.screenScale*100));
		quiteValueLabel.setText((int)(Config.screenScale*100)+"%");
		if(Config.startOnInit){
			isStartOnInitCheckBox.setSelected(true);
			serviceOperation();
		}
	}
	
	private class ControlPanel extends JPanel{
		private static final long serialVersionUID = 1L;
		public ControlPanel(){
			this.setLayout(null);
			
			JLabel lanIPLabel = new JLabel("内网IP：");
			lanIPLabel.setVerticalAlignment(SwingConstants.TOP);
			lanIPLabel.setLocation(5, 5);
			lanIPLabel.setSize(50, 20);
			this.add(lanIPLabel);

			lanIPValueLabel.setVerticalAlignment(SwingConstants.TOP);
			lanIPValueLabel.setLocation(55, 5);
			lanIPValueLabel.setSize(150, 60);
			this.add(lanIPValueLabel);
			
			startServiceBtn.setLocation(180, 45);
			startServiceBtn.setSize(100, 20);
			startServiceBtn.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent arg0) {
				serviceOperation();
			}});
			this.add(startServiceBtn);
			
			final JList list = new JList();
			list.setModel(model);
			JScrollPane listlPanel = new JScrollPane(list);
			listlPanel.setSize(280, 80);
			listlPanel.setLocation(5, 70);
			add(listlPanel);
		}
	}
	
	private void serviceOperation(){
		startServiceBtn.setEnabled(false);
		if(isServicing){ // 开始停止服务
			Program.listener.stopService();
			Program.listener=null;
			startServiceBtn.setText("启动服务");
			portValue.setEnabled(true);
//			isSingleUserCheckBox.setEnabled(true);
			sizeCheckBox.setEnabled(true);
			widthNumber.setEnabled(sizeCheckBox.isSelected());
			heightNumber.setEnabled(sizeCheckBox.isSelected());
			isServicing=false;
			if(Program.sr!=null){
				Program.sr.stop();
				Program.sr=null;
			}
		}else{ // 开始启动服务
			Program.listener = new Listener(Config.port);
			Program.listener.startService();
			startServiceBtn.setText("停止服务");
			portValue.setEnabled(false);
//			isSingleUserCheckBox.setEnabled(false);
			sizeCheckBox.setEnabled(false);
			widthNumber.setEnabled(false);
			heightNumber.setEnabled(false);
			isServicing=true;
//			if(Config.useSound){
//				Program.sr = new SoundRecorder();
//				Program.sr.addErrorListener(new SoundRecorder.ErrorListener() { public void onError(String errorMsg) {
//					
//				}});
//				Program.sr.start();
//			}
			Program.ScreenSource = new FileUtil().getText(Config.PATH_HTML, "UTF-8")
					.replace("{width}", String.valueOf(Config.showSrcWidth))
					.replace("{height}", String.valueOf(Config.showSrcHeight));
		}
		startServiceBtn.setEnabled(true);
	}
	
	private void trayOperation(){
		if(tray==null)
			return;
		tray.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if(!container.isVisible())
					container.setVisible(true);
				if(container instanceof JFrame)
					((JFrame)container).setState(Frame.NORMAL);
			}
		});
	}
	
	private class SysSettingPanel extends JPanel{
		private static final long serialVersionUID = 1L;
		public SysSettingPanel(){
			this.setLayout(null);
			
			JLabel portLabel = new JLabel("监听端口：");
			portLabel.setLocation(5, 5);
			portLabel.setSize(100, 20);
			this.add(portLabel);
			
			portValue.setLocation(110, 5);
			portValue.setSize(60, 20);
			portValue.setMinimumValue(1);
			portValue.setMaximumValue(65535);
			portValue.addValueChangeListener(new ValueChangeListener() { public void valueChanged(int value) {
				Config.port=portValue.getNumber();
			}});
			this.add(portValue);
			
//			isSoundCheckBox.setLocation(5, 30);
//			isSoundCheckBox.setSize(100, 20);
//			this.add(isSoundCheckBox);
			
//			JLabel shoudDeviceLabel = new JLabel("声音捕获设备：");
//			shoudDeviceLabel.setLocation(5, 55);
//			shoudDeviceLabel.setSize(100, 20);
//			this.add(shoudDeviceLabel);
			
			isStartOnInitCheckBox.setLocation(5, 30);
			isStartOnInitCheckBox.setSize(200, 20);
			isStartOnInitCheckBox.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent arg0) {
				Config.startOnInit=isStartOnInitCheckBox.isSelected();
			}});
			this.add(isStartOnInitCheckBox);
			
			isMinOnInitCheckBox.setLocation(5, 55);
			isMinOnInitCheckBox.setSize(200, 20);
			isMinOnInitCheckBox.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent arg0) {
				Config.miniOnInit=isMinOnInitCheckBox.isSelected();
			}});
			this.add(isMinOnInitCheckBox);
			
//			isSingleUserCheckBox.setLocation(5, 80);
//			isSingleUserCheckBox.setSize(200, 20);
//			isSingleUserCheckBox.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent arg0) {
//				Config.isSingleUser=isSingleUserCheckBox.isSelected();
//			}});
//			this.add(isSingleUserCheckBox);
			
			isShowInTaskCheckBox.setLocation(5, 80);
			isShowInTaskCheckBox.setSize(200, 20);
			isShowInTaskCheckBox.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent arg0) {
				Config.showInTask=isShowInTaskCheckBox.isSelected();
			}});
			isShowInTaskCheckBox.setSelected(true);
			isShowInTaskCheckBox.setEnabled(false); //TODO
			this.add(isShowInTaskCheckBox);
			
			isShowTrayCheckBox.setLocation(5, 105);
			isShowTrayCheckBox.setSize(200, 20);
			isShowTrayCheckBox.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent arg0) {
				Config.useTray=isShowTrayCheckBox.isSelected();
				if(Config.useTray){
					if(tray!=null)
						return;
					tray = RuntimeUtil.addTrayIcon(new File(Config.PATH_LOGO));
					trayOperation();
				}else{
					RuntimeUtil.removeTrayIcon(tray);
					tray=null;
				}
			}});
			isShowTrayCheckBox.setSelected(true);
			isShowTrayCheckBox.setEnabled(false); // TODO
			this.add(isShowTrayCheckBox);
			
			JLabel closeLabel = new JLabel("关闭窗口时的动作：");
			closeLabel.setLocation(5, 130);
			closeLabel.setSize(120, 20);
			this.add(closeLabel);
			
			closeCombox.setLocation(125, 130);
			closeCombox.setSize(100, 20);
			closeCombox.addItemListener(new ItemListener() { public void itemStateChanged(ItemEvent e) {
				Config.actionOnClose=closeCombox.getSelectedIndex();
			}});
			this.add(closeCombox);
		}
	}
	
	private class ViewSettingPanel extends JPanel{
		private static final long serialVersionUID = 1L;
		public ViewSettingPanel(){
			this.setLayout(null);
			
			sizeCheckBox.setLocation(5, 5);
			sizeCheckBox.setSize(145, 20);
			sizeCheckBox.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent arg0) {
				Config.aotuFit=!sizeCheckBox.isSelected();
				widthNumber.setEnabled(sizeCheckBox.isSelected());
				heightNumber.setEnabled(sizeCheckBox.isSelected());
			}});
			this.add(sizeCheckBox);
			
			widthNumber.setLocation(150, 5);
			widthNumber.setSize(50, 20);
			widthNumber.setMaximumValue(99999);
			widthNumber.setMinimumValue(100);
			widthNumber.addValueChangeListener(new ValueChangeListener() { public void valueChanged(int value) {
				Config.showSrcWidth=value;
			}});
			this.add(widthNumber);
			
			JLabel lLabel = new JLabel("X");
			lLabel.setLocation(210, 5);
			lLabel.setSize(20, 20);
			this.add(lLabel);
			
			heightNumber.setLocation(230, 5);
			heightNumber.setSize(50, 20);
			heightNumber.setMaximumValue(99999);
			heightNumber.setMinimumValue(100);
			heightNumber.addValueChangeListener(new ValueChangeListener() { public void valueChanged(int value) {
				Config.showSrcHeight=value;
			}});
			this.add(heightNumber);
			
			JLabel quiteLabel = new JLabel("清晰度：");
			quiteLabel.setLocation(10, 40);
			quiteLabel.setSize(50, 20);
			this.add(quiteLabel);
			
			queSlider.setLocation(60, 40);
			queSlider.setSize(200, 20);
			queSlider.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent e) {
				Config.screenScale=((double)queSlider.getValue())/100;
				quiteValueLabel.setText(queSlider.getValue()+"%");
			}});
			this.add(queSlider);
			
			quiteValueLabel.setLocation(260, 40);
			quiteValueLabel.setSize(100, 20);
			this.add(quiteValueLabel);
			
//			JLabel cutLabel = new JLabel("屏幕裁剪：");
//			cutLabel.setLocation(10, 55);
//			cutLabel.setSize(100, 20);
//			this.add(cutLabel);
			
			isViewOnlyCheckBox.setLocation(5, 75);
			isViewOnlyCheckBox.setSize(200, 20);
			isViewOnlyCheckBox.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent arg0) {
				Config.isViewMode=isViewOnlyCheckBox.isSelected();
			}});
			this.add(isViewOnlyCheckBox);
		}
	}
	
	private class DownloadPanel extends JPanel{
		private static final long serialVersionUID = 1L;
		public DownloadPanel(){
			this.setLayout(new BorderLayout());
			infoLabel.setHorizontalAlignment(SwingConstants.CENTER);
			infoLabel.setVerticalAlignment(SwingConstants.CENTER);
			this.add(infoLabel, BorderLayout.CENTER);
			new DropTarget(infoLabel, DnDConstants.ACTION_COPY_OR_MOVE, new DropTargetAdapter() { public void drop(DropTargetDropEvent dtde) {
				try {
	                if (dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
	                    dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
	                    @SuppressWarnings("unchecked")
						List<File> list = (List<File>)(dtde.getTransferable().getTransferData(DataFlavor.javaFileListFlavor));
	                    if(list==null || list.size()==0)
	                    	return;
	                    File file = list.get(0);
	                    if(!file.exists() || file.isDirectory())
	                    	return;
	                    dlPath=file.getAbsolutePath();
	                    infoLabel.setText(dlPath.length()>20?dlPath.substring(0, 10)+"..."+dlPath.substring(dlPath.length()-9):dlPath);
	                    dtde.dropComplete(true);
	                    infoLabel.updateUI();
	                }else {
	                    dtde.rejectDrop();
	                }
	            } catch (Exception e) { }
			}});
		}
	}
	
	private class AboutPanel extends JPanel{
		private static final long serialVersionUID = 1L;
		public AboutPanel(){
			this.setLayout(new BorderLayout());
			
			JLabel helpLabel = new JLabel();
			helpLabel.setHorizontalAlignment(SwingConstants.LEFT);
			helpLabel.setVerticalAlignment(SwingConstants.CENTER);
			String helpInfo = "<html>"+new FileUtil().getText(Config.PATH_HELP, "UTF-8")+"</html>";
			helpInfo=helpInfo.replace("\r", "").replace("\n", "<br>");
			helpLabel.setText(helpInfo);
			
			JScrollPane scrollPane=new JScrollPane(helpLabel);
			this.add(scrollPane, BorderLayout.CENTER);
		}
	}
	
	private class InfoPanel extends JPanel{
		private static final long serialVersionUID = 1L;
		public InfoPanel(){
			this.setLayout(new BorderLayout());
			
			JLabel helpLabel = new JLabel("<html>"+
					Config.AppTitle+"<br>"+
					"<Strong>Version:</Strong> 2.0.13.1030<br><br>"+
					"Copyright@2012-2013 Wafly.<br>"+
					"All Rights Reserved.<br><br>"+
					"<Strong>Support:</Strong> wafly@qq.com<br>"+
					"<Strong>Online Support:</Strong> 298039945(QQ)<br>"+
					"<Strong>Web Page:</Strong> <a>http://blog.csdn.net/wafly</a>"+
					"</html>");
			helpLabel.setHorizontalAlignment(SwingConstants.LEFT);
			helpLabel.setVerticalAlignment(SwingConstants.CENTER);
			
			JScrollPane scrollPane=new JScrollPane(helpLabel);
			this.add(scrollPane, BorderLayout.CENTER);
		}
	}
}
